import { __awaiter } from 'tslib';
import * as i0 from '@angular/core';
import { Directive, Input, EventEmitter, Component, Output, ViewChild, NgModule, Injectable } from '@angular/core';
import { ComponentMapping } from '@adobe/aem-spa-component-mapping';
import { Constants as Constants$1, ModelManager, PathUtils } from '@adobe/aem-spa-page-model-manager';
import * as i2 from '@angular/common';
import { CommonModule } from '@angular/common';

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
/**
 * Provides standard implementation for the MappedComponentProperties using @Input
 */
class AbstractMappedComponentDirective {
    constructor() {
        this.isInEditor = false;
        this.cqPath = '';
        this.itemName = '';
    }
}
AbstractMappedComponentDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AbstractMappedComponentDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
AbstractMappedComponentDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.4.0", type: AbstractMappedComponentDirective, inputs: { isInEditor: "isInEditor", cqPath: "cqPath", itemName: "itemName" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AbstractMappedComponentDirective, decorators: [{
            type: Directive
        }], propDecorators: { isInEditor: [{
                type: Input
            }], cqPath: [{
                type: Input
            }], itemName: [{
                type: Input
            }] } });
/**
 * The current class extends the @adobe/cq-spa-component-mapping#Mapto library and features with Angular specifics such as
 *
 * - Storing the editing configurations for each resource type
 */
class ComponentMappingWithConfig {
    constructor(spaMapping) {
        this.spaMapping = spaMapping;
        /**
         * Store of EditConfig structures
         */
        this.editConfigMap = {};
    }
    /**
     * Stores a component class for the given resource types and also allows to provide an EditConfig object
     * @param resourceTypes - List of resource types
     * @param clazz - Component class to be stored
     * @param [editConfig] - Edit configuration to be stored for the given resource types
     * @type Model - The Model interface / class type bound to the editconfig object.
     */
    map(resourceTypes, clazz, editConfig = null) {
        const innerClass = clazz;
        const resourceList = (typeof resourceTypes === 'string') ? [resourceTypes] : resourceTypes;
        resourceList.forEach((entry) => {
            if (editConfig) {
                this.editConfigMap[entry] = editConfig;
            }
            this.spaMapping.map(entry, innerClass);
        });
    }
    /**
     * Stores a  component class for the given resource types and also allows to provide an EditConfig object in a Lazy Manner
     * @param resourceTypes - List of resource types
     * @param lazyClassFunction - A function that returns a promise to give back the designated type / class
     * @param [editConfig] - Edit configuration to be stored for the given resource types
     * @type Model - The Model interface / class type bound to the editconfig object.
     */
    lazyMap(resourceTypes, lazyClassFunction, editConfig = null) {
        const innerFunction = lazyClassFunction;
        if (editConfig) {
            this.editConfigMap[resourceTypes] = editConfig;
        }
        this.spaMapping.lazyMap(resourceTypes, innerFunction);
    }
    /**
     * Returns the component class for the given resourceType
     * @param resourceType - Resource type for which the component class has been stored
     * @type Model - The Model interface / class type bound to the editconfig object.
     */
    get(resourceType) {
        return this.spaMapping.get(resourceType);
    }
    /**
       * Returns the component class Promise for the given resourceType
       * @param resourceType - Resource type for which the component class has been stored
       * @type Model - The Model interface / class type bound to the editconfig object.
       */
    lazyGet(resourceType) {
        return this.spaMapping.getLazy(resourceType);
    }
    /**
     * Returns the EditConfig structure for the given type
     * @param resourceType - Resource type for which the configuration has been stored
     * @type Model - The Model interface / class type bound to the editconfig object.
     */
    getEditConfig(resourceType) {
        return this.editConfigMap[resourceType];
    }
}
const componentMapping = new ComponentMappingWithConfig(ComponentMapping);
/**
 * Stores a component class for the given resource types and also allows to provide an EditConfig object
 * @param resourceTypes - List of resource types
 * @type Model - The Model interface / class type that will be Mapped. Bound to the EditConfig configuration.
 */
function MapTo(resourceTypes) {
    /**
     * @param clazz - Component class to be stored
     * @param [editConfig] - Edit configuration to be stored for the given resource types
     */
    return (clazz, editConfig = null) => componentMapping.map(resourceTypes, clazz, editConfig);
}
/**
 * Stores a clazz the lazy way for dynamic imports / code splitting.function that returns a promise
 * @param resourceTypes - List of resource types
 * @type Model - The Model interface / class type that will be Mapped. Bound to the EditConfig configuration.
 */
function LazyMapTo(resourceTypes) {
    /**
     * @param lazyClassPromise - Function that returns a promise resolving a class
     * @param [editConfig] - Edit configuration to be stored for the given resource types
     */
    return (lazyClassFunction, editConfig = null) => componentMapping.lazyMap(resourceTypes, lazyClassFunction, editConfig);
}

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
const Constants = {
    /**
     * Class names associated with a new section component
     *
     */
    NEW_SECTION_CLASS_NAMES: 'new section',
    /**
     *  CSS Class names applied to a component.
     */
    APPLIED_CLASS_NAMES: 'appliedCssClassNames',
    TYPE_PROP: Constants$1.TYPE_PROP,
    /**
     * List of child items of an item
     *
     */
    ITEMS_PROP: Constants$1.ITEMS_PROP,
    /**
     * Order in which the items should be listed
     *
     */
    ITEMS_ORDER_PROP: Constants$1.ITEMS_ORDER_PROP,
    /**
     * Path of the item
     *
     */
    PATH_PROP: Constants$1.PATH_PROP,
    /**
     * Children of an item
     *
     */
    CHILDREN_PROP: Constants$1.CHILDREN_PROP,
    /**
     * Path of the resource in the model
     *
     */
    DATA_PATH_PROP: ':dataPath',
    /**
     * Hierarchical type of the item
     */
    HIERARCHY_TYPE_PROP: Constants$1.HIERARCHY_TYPE_PROP,
    /**
     * Event which indicates that content of remote component has been fetched and loaded in the app
     */
    ASYNC_CONTENT_LOADED_EVENT: 'cq-async-content-loaded'
};

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
/**
 * Selector that identifies the node that contains the WCM mode state.
 * @private
 */
const WCM_MODE_META_SELECTOR = 'meta[property="cq:wcmmode"]';
/**
 * The editor is in one of the edition modes.
 * @private
 */
const EDIT_MODE = 'edit';
/**
 * The editor is in preview mode.
 * @private
 */
const PREVIEW_MODE = 'preview';
/**
 * Returns if we are in the browser context or not by checking for the
 * existence of the window object.
 * @private
 */
function isBrowser() {
    try {
        return typeof window !== 'undefined';
    }
    catch (e) {
        return false;
    }
}
/**
 * Returns the current WCM mode
 *
 * <p>Note that the value isn't, as of the date of this writing, updated by the editor</p>
 * @private
 */
function getWCMMode() {
    if (isBrowser()) {
        const wcmModeMeta = document.head.querySelector(WCM_MODE_META_SELECTOR);
        return wcmModeMeta && wcmModeMeta.content;
    }
}
/**
 * Helper functions for interacting with the AEM environment.
 */
const Utils = {
    /**
     * Is the app used in the context of the AEM Page editor.
     */
    isInEditor() {
        const wcmMode = getWCMMode();
        return wcmMode && (EDIT_MODE === wcmMode || PREVIEW_MODE === wcmMode);
    },
    /**
     * Determines the cqPath of a component given its props
     *
     * @private
     * @returns cqPath of the component
     */
    getCQPath(pagePath, itemPath) {
        let path = (itemPath ? `${pagePath}/jcr:content/${itemPath}` : pagePath);
        path = path.replace(/\/+/g, '/');
        return path;
    }
};

/**
 * @private
 */
const PLACEHOLDER_CLASS_NAME = 'cq-placeholder';
/**
 * The current directive provides advanced capabilities among which are
 *
 * - The management of the component placeholder in the Page Editor
 * - The dynamic instantiation of components based on a component definition
 * - The conversion from model fields to properties and injection in the component instance
 * - The management of HTMLElement attributes and class names on the native element
 */
class AEMComponentDirective {
    constructor(renderer, viewContainer, compiler, injector, factoryResolver, _changeDetectorRef) {
        this.renderer = renderer;
        this.viewContainer = viewContainer;
        this.compiler = compiler;
        this.injector = injector;
        this.factoryResolver = factoryResolver;
        this._changeDetectorRef = _changeDetectorRef;
    }
    get cqItem() {
        return this._cqItem;
    }
    set cqItem(value) {
        this._cqItem = value;
    }
    get changeDetectorRef() {
        return this._changeDetectorRef;
    }
    ngOnInit() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.type) {
                const mappedFn = componentMapping.get(this.type);
                if (mappedFn) {
                    this.renderComponent(mappedFn);
                }
                else {
                    yield this.initializeAsync();
                }
            }
            else {
                console.warn('no type on ' + this.cqPath);
            }
        });
    }
    initializeAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            const lazyMappedPromise = componentMapping.lazyGet(this.type);
            try {
                const LazyResolvedComponent = yield lazyMappedPromise;
                this.renderComponent(LazyResolvedComponent);
                this.loaded = true;
                this._changeDetectorRef.detectChanges();
            }
            catch (err) {
                console.warn(err);
            }
        });
    }
    ngOnChanges() {
        this.updateComponentData();
    }
    /**
     * Returns the type of the cqItem if exists.
     */
    get type() {
        return this.cqItem && this.cqItem[Constants.TYPE_PROP];
    }
    /**
     * Renders a component dynamically based on the component definition
     *
     * @param componentDefinition The component definition to render
     */
    renderComponent(componentDefinition) {
        if (componentDefinition) {
            const factory = this.factoryResolver.resolveComponentFactory(componentDefinition);
            this.renderWithFactory(factory);
        }
        else {
            throw new Error('No component definition!!');
        }
    }
    renderWithFactory(factory) {
        this.viewContainer.clear();
        this._component = this.viewContainer.createComponent(factory);
        this.updateComponentData();
    }
    /**
     * Updates the data of the component based the data of the directive
     */
    updateComponentData() {
        if (!this._component || !this._component.instance || !this.cqItem) {
            return;
        }
        const keys = Object.getOwnPropertyNames(this.cqItem);
        keys.forEach((key) => {
            let propKey = key;
            if (propKey.startsWith(':')) {
                // Transformation of internal properties namespaced with [:] to [cq]
                // :myProperty => cqMyProperty
                const tempKey = propKey.substr(1);
                propKey = 'cq' + tempKey.substr(0, 1).toUpperCase() + tempKey.substr(1);
            }
            this._component.instance[propKey] = this.cqItem[key];
        });
        this._component.instance.cqPath = this.cqPath;
        this._component.instance.itemName = this.itemName || (this.cqItem && this.cqItem.id);
        this.includeAppliedCSSClasses();
        const editConfig = componentMapping.getEditConfig(this.type);
        if (editConfig && Utils.isInEditor) {
            this.setupPlaceholder(editConfig);
        }
        this._changeDetectorRef.detectChanges();
    }
    /**
     * Adds the applied css class names in to the element
     */
    includeAppliedCSSClasses() {
        const appliedCssClassNames = this.cqItem[Constants.APPLIED_CLASS_NAMES] || '';
        if (appliedCssClassNames && this._component) {
            this.renderer.setAttribute(this._component.location.nativeElement, 'class', appliedCssClassNames);
        }
    }
    /**
     * Adds the specified item attributes to the element
     */
    setupItemAttrs() {
        if (this.itemAttrs) {
            const keys = Object.getOwnPropertyNames(this.itemAttrs);
            keys.forEach((key) => {
                if (key === 'class') {
                    const classes = this.itemAttrs[key].split(' ');
                    classes.forEach((itemClass) => {
                        this.renderer.addClass(this._component.location.nativeElement, itemClass);
                    });
                }
                else {
                    this.renderer.setAttribute(this._component.location.nativeElement, key, this.itemAttrs[key]);
                }
            });
        }
    }
    /**
     * Determines if the placeholder should e displayed.
     *
     * @param editConfig - the edit config of the directive
     */
    usePlaceholder(editConfig) {
        return editConfig.isEmpty && typeof editConfig.isEmpty === 'function' && editConfig.isEmpty(this.cqItem);
    }
    /**
     * Setups the placeholder of needed for the AEM editor
     *
     * @param editConfig - the editConfig, which will dictate the classes to be added on.
     */
    setupPlaceholder(editConfig) {
        if (this.usePlaceholder(editConfig)) {
            this.renderer.addClass(this._component.location.nativeElement, PLACEHOLDER_CLASS_NAME);
            this.renderer.setAttribute(this._component.location.nativeElement, 'data-emptytext', editConfig.emptyLabel);
        }
        else {
            this.renderer.removeClass(this._component.location.nativeElement, PLACEHOLDER_CLASS_NAME);
            this.renderer.removeAttribute(this._component.location.nativeElement, 'data-emptytext');
        }
    }
    ngAfterViewInit() {
        this.setupItemAttrs();
    }
    ngOnDestroy() {
        if (this._component) {
            this._component.destroy();
        }
    }
}
AEMComponentDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMComponentDirective, deps: [{ token: i0.Renderer2 }, { token: i0.ViewContainerRef }, { token: i0.Compiler }, { token: i0.Injector }, { token: i0.ComponentFactoryResolver }, { token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Directive });
AEMComponentDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.4.0", type: AEMComponentDirective, selector: "[aemComponent]", inputs: { cqItem: "cqItem", cqPath: "cqPath", itemName: "itemName", itemAttrs: "itemAttrs", loaded: "loaded", aemComponent: "aemComponent" }, usesOnChanges: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMComponentDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[aemComponent]'
                }]
        }], ctorParameters: function () { return [{ type: i0.Renderer2 }, { type: i0.ViewContainerRef }, { type: i0.Compiler }, { type: i0.Injector }, { type: i0.ComponentFactoryResolver }, { type: i0.ChangeDetectorRef }]; }, propDecorators: { cqItem: [{
                type: Input
            }], cqPath: [{
                type: Input
            }], itemName: [{
                type: Input
            }], itemAttrs: [{
                type: Input
            }], loaded: [{
                type: Input
            }], aemComponent: [{
                type: Input
            }] } });

/**
 * The current component is responsible for providing access to the ModelManager and the model of a component
 */
class AEMModelProviderComponent {
    constructor() {
        this.updateDataPath = new EventEmitter();
    }
    /**
     * Updates the item data
     */
    updateItem() {
        ModelManager.getData({ path: this.cqPath }).then(model => {
            this.cqItemLoaded = Promise.resolve(true);
            this.cqItem = model;
            if (this.pagePath && Utils.isInEditor()) {
                PathUtils.dispatchGlobalCustomEvent(Constants.ASYNC_CONTENT_LOADED_EVENT, {});
            }
            if (this.aemComponent) {
                this.aemComponent.changeDetectorRef.markForCheck();
            }
        });
    }
    ngOnInit() {
        return __awaiter(this, void 0, void 0, function* () {
            yield ModelManager.initialize();
            if (!this.cqItem && this.pagePath) {
                this.cqPath = Utils.getCQPath(this.pagePath, this.itemPath);
                this.updateDataPath.emit({ cqPath: this.cqPath });
                this.updateItem();
            }
            else {
                this.cqItemLoaded = Promise.resolve(true);
            }
            ModelManager.addListener(this.cqPath, this.updateItem.bind(this));
        });
    }
    ngDestroy() {
        ModelManager.removeListener(this.cqPath, this.updateItem.bind(this));
    }
}
AEMModelProviderComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMModelProviderComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
AEMModelProviderComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.4.0", type: AEMModelProviderComponent, selector: "aem-model-provider,[aemModelProvider]", inputs: { cqPath: "cqPath", cqItem: "cqItem", itemName: "itemName", aemModelProvider: "aemModelProvider", pagePath: "pagePath", itemPath: "itemPath" }, outputs: { updateDataPath: "updateDataPath" }, viewQueries: [{ propertyName: "aemComponent", first: true, predicate: AEMComponentDirective, descendants: true }], ngImport: i0, template: `<ng-container *ngIf="cqItemLoaded | async" aemComponent [cqItem]='cqItem' [cqPath]='cqPath' [itemName]='itemName'></ng-container>`, isInline: true, directives: [{ type: i2.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { type: AEMComponentDirective, selector: "[aemComponent]", inputs: ["cqItem", "cqPath", "itemName", "itemAttrs", "loaded", "aemComponent"] }], pipes: { "async": i2.AsyncPipe } });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMModelProviderComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'aem-model-provider,[aemModelProvider]',
                    template: `<ng-container *ngIf="cqItemLoaded | async" aemComponent [cqItem]='cqItem' [cqPath]='cqPath' [itemName]='itemName'></ng-container>`
                }]
        }], ctorParameters: function () { return []; }, propDecorators: { cqPath: [{
                type: Input
            }], cqItem: [{
                type: Input
            }], itemName: [{
                type: Input
            }], aemModelProvider: [{
                type: Input
            }], pagePath: [{
                type: Input
            }], itemPath: [{
                type: Input
            }], updateDataPath: [{
                type: Output
            }], aemComponent: [{
                type: ViewChild,
                args: [AEMComponentDirective]
            }] } });

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
/**
 * @private
 */
const PLACEHOLDER_CLASS_NAMES$1 = Constants.NEW_SECTION_CLASS_NAMES;
/**
 * @private
 */
const PLACEHOLDER_ITEM_NAME = '*';
/**
 * @private
 */
const CONTAINER_CLASS_NAMES = 'aem-container';
/**
 * The current component provides the base presentational logic common to containers such as a grid or a page.
 * Container have in common the notion of item holders. Items are represented in the model by the fields _:items_ and _:itemsOrder_
 */
class AEMContainerComponent extends AbstractMappedComponentDirective {
    constructor() {
        super(...arguments);
        /**
         * Key of the model structure
         */
        this.modelName = '';
    }
    /**
     * Returns weather of not we are in the editor
     */
    get isInEditMode() {
        return Utils.isInEditor();
    }
    /**
     * Returns the aggregated path of this container path and the provided path
     *
     * @param path - the provided path to aggregate with the container path
     */
    getDataPath(path) {
        return this.cqPath ? this.cqPath + '/' + path : path;
    }
    /**
     * Returns the item data from the cqModel
     *
     * @param itemKey - the itemKey to look for in the items.
     */
    getItem(itemKey) {
        return this.cqItems && this.cqItems[itemKey];
    }
    /**
     * Returns the class names of the container based on the data from the cqModel
     */
    getHostClassNames() {
        return CONTAINER_CLASS_NAMES;
    }
    get hostClasses() {
        return this.getHostClassNames();
    }
    /**
     * Returns the placeholder classes
     */
    getPlaceholderClassNames() {
        return PLACEHOLDER_CLASS_NAMES$1;
    }
    /**
     * Returns the placeholder path
     */
    get placeholderPath() {
        return this.cqPath && this.cqPath + '/' + PLACEHOLDER_ITEM_NAME;
    }
}
AEMContainerComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMContainerComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
AEMContainerComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.4.0", type: AEMContainerComponent, selector: "aem-container", inputs: { cqItems: "cqItems", cqItemsOrder: "cqItemsOrder", classNames: "classNames", modelName: "modelName" }, host: { properties: { "class": "hostClasses", "attr.data-cq-data-path": "cqPath" } }, usesInheritance: true, ngImport: i0, template: "<ng-container *ngFor=\"let itemKey of cqItemsOrder\">\n  <aem-model-provider [cqItem]=\"getItem(itemKey)\"\n                      [cqPath]=\"getDataPath(itemKey)\"\n                      [itemName]=\"itemKey\"></aem-model-provider>\n</ng-container>\n<div *ngIf=\"isInEditMode\"\n     [attr.data-cq-data-path]=\"placeholderPath\"\n     [class]=\"getPlaceholderClassNames()\"></div>\n", components: [{ type: AEMModelProviderComponent, selector: "aem-model-provider,[aemModelProvider]", inputs: ["cqPath", "cqItem", "itemName", "aemModelProvider", "pagePath", "itemPath"], outputs: ["updateDataPath"] }], directives: [{ type: i2.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { type: i2.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMContainerComponent, decorators: [{
            type: Component,
            args: [{ selector: 'aem-container', host: {
                        '[class]': 'hostClasses',
                        '[attr.data-cq-data-path]': 'cqPath'
                    }, template: "<ng-container *ngFor=\"let itemKey of cqItemsOrder\">\n  <aem-model-provider [cqItem]=\"getItem(itemKey)\"\n                      [cqPath]=\"getDataPath(itemKey)\"\n                      [itemName]=\"itemKey\"></aem-model-provider>\n</ng-container>\n<div *ngIf=\"isInEditMode\"\n     [attr.data-cq-data-path]=\"placeholderPath\"\n     [class]=\"getPlaceholderClassNames()\"></div>\n" }]
        }], propDecorators: { cqItems: [{
                type: Input
            }], cqItemsOrder: [{
                type: Input
            }], classNames: [{
                type: Input
            }], modelName: [{
                type: Input
            }] } });

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
/**
 * @private
 */
const ALLOWED_PLACEHOLDER_CLASS_NAMES = 'aem-AllowedComponent--list';
/**
 * @private
 */
const ALLOWED_COMPONENT_TITLE_CLASS_NAMES = 'aem-AllowedComponent--title';
/**
 * @private
 */
const ALLOWED_COMPONENT_PLACEHOLDER_CLASS_NAMES = 'aem-AllowedComponent--component cq-placeholder placeholder';
class AEMAllowedComponentsContainerComponent extends AEMContainerComponent {
    constructor() {
        super(...arguments);
        this.emptyLabel = 'No allowed components';
    }
    isAllowedComponentsApplicable() {
        return this.isInEditMode && this.allowedComponents && this.allowedComponents.applicable;
    }
    getAllowedComponentListPlaceholderClassNames() {
        return super.getPlaceholderClassNames() + ' ' + ALLOWED_PLACEHOLDER_CLASS_NAMES;
    }
    getAllowedComponentListLabel() {
        const hasComponents = this.allowedComponents && this.allowedComponents.components && this.allowedComponents.components.length > 0;
        return hasComponents ? this.title : this.emptyLabel;
    }
    getAllowedComponents() {
        return this.allowedComponents && this.allowedComponents.components || [];
    }
    get allowedComponentListTitleClassNames() {
        return ALLOWED_COMPONENT_TITLE_CLASS_NAMES;
    }
    get allowedComponentClassNames() {
        return ALLOWED_COMPONENT_PLACEHOLDER_CLASS_NAMES;
    }
}
AEMAllowedComponentsContainerComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMAllowedComponentsContainerComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
AEMAllowedComponentsContainerComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.4.0", type: AEMAllowedComponentsContainerComponent, selector: "aem-allowed-components-container", inputs: { title: "title", emptyLabel: "emptyLabel", allowedComponents: "allowedComponents" }, usesInheritance: true, ngImport: i0, template: "<div [class]=\"getAllowedComponentListPlaceholderClassNames()\">\n  <div [attr.data-text]=\"getAllowedComponentListLabel()\" [class]=\"allowedComponentListTitleClassNames\"></div>\n  <ng-container *ngFor=\"let allowedComponent of getAllowedComponents()\">\n    <div [attr.data-cq-data-path]=\"allowedComponent.path\"\n         [attr.data-emptytext]=\"allowedComponent.title\"\n         [class]=\"allowedComponentClassNames\"></div>\n  </ng-container>\n</div>\n", styles: [""], directives: [{ type: i2.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMAllowedComponentsContainerComponent, decorators: [{
            type: Component,
            args: [{ selector: 'aem-allowed-components-container', template: "<div [class]=\"getAllowedComponentListPlaceholderClassNames()\">\n  <div [attr.data-text]=\"getAllowedComponentListLabel()\" [class]=\"allowedComponentListTitleClassNames\"></div>\n  <ng-container *ngFor=\"let allowedComponent of getAllowedComponents()\">\n    <div [attr.data-cq-data-path]=\"allowedComponent.path\"\n         [attr.data-emptytext]=\"allowedComponent.title\"\n         [class]=\"allowedComponentClassNames\"></div>\n  </ng-container>\n</div>\n", styles: [""] }]
        }], propDecorators: { title: [{
                type: Input
            }], emptyLabel: [{
                type: Input
            }], allowedComponents: [{
                type: Input
            }] } });

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
/**
 * @private
 */
const PLACEHOLDER_CLASS_NAMES = 'aem-Grid-newComponent';
/**
 * @private
 */
const RESPONSIVE_GRID_TYPE = 'wcm/foundation/components/responsivegrid';
/**
 * The current class carries the base presentational logic of the AEM Layout Container (aka. Responsive grid)
 */
class AEMResponsiveGridComponent extends AEMAllowedComponentsContainerComponent {
    /**
     * Returns the column class names for a given column
     * @param itemKey - The key of the column item
     */
    getColumnClassNames(itemKey) {
        return this.columnClassNames && this.columnClassNames[itemKey];
    }
    /**
     * Returns the placeholder classes
     */
    getPlaceholderClassNames() {
        return super.getPlaceholderClassNames() + ' ' + PLACEHOLDER_CLASS_NAMES;
    }
    /**
     * Returns the class names of the responsive grid based on the data from the cqModel
     */
    getHostClassNames() {
        let classNames = super.getHostClassNames();
        if (this.classNames) {
            classNames += ' ' + (this.classNames || '');
        }
        return classNames + ' ' + this.gridClassNames;
    }
    /**
     * Returns the aggregated path of this container path and the provided path
     *
     * @param path - the provided path to aggregate with the container path
     */
    getAttrDataPath(path) {
        const item = this.getItem(path);
        if (item && item[Constants.TYPE_PROP] === RESPONSIVE_GRID_TYPE) {
            // We don't want to add the path for the wrapper for a reponsivegrid
            // The reponsivegrid adds the path on it's own
            return null;
        }
        return this.getDataPath(path);
    }
}
AEMResponsiveGridComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMResponsiveGridComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
AEMResponsiveGridComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.4.0", type: AEMResponsiveGridComponent, selector: "aem-responsivegrid", inputs: { gridClassNames: "gridClassNames", columnClassNames: "columnClassNames", classNames: "classNames", columnCount: "columnCount" }, host: { properties: { "class": "hostClasses", "attr.data-cq-data-path": "cqPath" } }, usesInheritance: true, ngImport: i0, template: "<ng-container *ngIf=\"isAllowedComponentsApplicable(); else container\">\n  <div [class]=\"getAllowedComponentListPlaceholderClassNames()\">\n    <div [attr.data-text]=\"getAllowedComponentListLabel()\" [class]=\"allowedComponentListTitleClassNames\"></div>\n    <ng-container *ngFor=\"let allowedComponent of getAllowedComponents()\">\n      <div [attr.data-cq-data-path]=\"allowedComponent.path\"\n           [attr.data-emptytext]=\"allowedComponent.title\"\n           [class]=\"allowedComponentClassNames\"></div>\n    </ng-container>\n  </div>\n</ng-container>\n<ng-template #container>\n  <ng-container *ngFor=\"let itemKey of cqItemsOrder\">\n    <div [aemModelProvider]\n         [cqItem]=\"getItem(itemKey)\"\n         [cqPath]=\"getDataPath(itemKey)\"\n         [itemName]=\"itemKey\"\n         [class]=\"getColumnClassNames(itemKey)\"\n         [attr.data-cq-data-path]=\"getAttrDataPath(itemKey)\"></div>\n  </ng-container>\n  <div *ngIf=\"isInEditMode\"\n       [attr.data-cq-data-path]=\"placeholderPath\"\n       [class]=\"getPlaceholderClassNames()\"></div>\n</ng-template>\n\n\n", components: [{ type: AEMModelProviderComponent, selector: "aem-model-provider,[aemModelProvider]", inputs: ["cqPath", "cqItem", "itemName", "aemModelProvider", "pagePath", "itemPath"], outputs: ["updateDataPath"] }], directives: [{ type: i2.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { type: i2.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMResponsiveGridComponent, decorators: [{
            type: Component,
            args: [{ selector: 'aem-responsivegrid', host: {
                        '[class]': 'hostClasses',
                        '[attr.data-cq-data-path]': 'cqPath'
                    }, template: "<ng-container *ngIf=\"isAllowedComponentsApplicable(); else container\">\n  <div [class]=\"getAllowedComponentListPlaceholderClassNames()\">\n    <div [attr.data-text]=\"getAllowedComponentListLabel()\" [class]=\"allowedComponentListTitleClassNames\"></div>\n    <ng-container *ngFor=\"let allowedComponent of getAllowedComponents()\">\n      <div [attr.data-cq-data-path]=\"allowedComponent.path\"\n           [attr.data-emptytext]=\"allowedComponent.title\"\n           [class]=\"allowedComponentClassNames\"></div>\n    </ng-container>\n  </div>\n</ng-container>\n<ng-template #container>\n  <ng-container *ngFor=\"let itemKey of cqItemsOrder\">\n    <div [aemModelProvider]\n         [cqItem]=\"getItem(itemKey)\"\n         [cqPath]=\"getDataPath(itemKey)\"\n         [itemName]=\"itemKey\"\n         [class]=\"getColumnClassNames(itemKey)\"\n         [attr.data-cq-data-path]=\"getAttrDataPath(itemKey)\"></div>\n  </ng-container>\n  <div *ngIf=\"isInEditMode\"\n       [attr.data-cq-data-path]=\"placeholderPath\"\n       [class]=\"getPlaceholderClassNames()\"></div>\n</ng-template>\n\n\n" }]
        }], propDecorators: { gridClassNames: [{
                type: Input
            }], columnClassNames: [{
                type: Input
            }], classNames: [{
                type: Input
            }], columnCount: [{
                type: Input
            }] } });

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
/**
 * @private
 */
const PAGE_MODEL_SEPARATOR = '/jcr:content/';
/**
 * The current component carries the base presentational logic of page component
 */
class AEMPageComponent extends AEMContainerComponent {
    /**
     * Returns the aggregated path of this container path and the provided path
     *
     * @param path - the provided path to aggregate with the container path
     */
    getDataPath(path) {
        return this.cqPath ? this.cqPath + PAGE_MODEL_SEPARATOR + path : path;
    }
}
AEMPageComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMPageComponent, deps: null, target: i0.ɵɵFactoryTarget.Component });
AEMPageComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.4.0", type: AEMPageComponent, selector: "aem-page", host: { properties: { "class": "hostClasses", "attr.data-cq-data-path": "cqPath" } }, usesInheritance: true, ngImport: i0, template: "<ng-container *ngFor=\"let itemKey of cqItemsOrder\">\n  <aem-model-provider [cqItem]=\"getItem(itemKey)\"\n                      [cqPath]=\"getDataPath(itemKey)\"\n                      [itemName]=\"itemKey\"></aem-model-provider>\n</ng-container>\n<div *ngIf=\"isInEditMode\"\n     [attr.data-cq-data-path]=\"placeholderPath\"\n     [class]=\"getPlaceholderClassNames()\"></div>\n", components: [{ type: AEMModelProviderComponent, selector: "aem-model-provider,[aemModelProvider]", inputs: ["cqPath", "cqItem", "itemName", "aemModelProvider", "pagePath", "itemPath"], outputs: ["updateDataPath"] }], directives: [{ type: i2.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { type: i2.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMPageComponent, decorators: [{
            type: Component,
            args: [{ selector: 'aem-page', host: {
                        '[class]': 'hostClasses',
                        '[attr.data-cq-data-path]': 'cqPath'
                    }, template: "<ng-container *ngFor=\"let itemKey of cqItemsOrder\">\n  <aem-model-provider [cqItem]=\"getItem(itemKey)\"\n                      [cqPath]=\"getDataPath(itemKey)\"\n                      [itemName]=\"itemKey\"></aem-model-provider>\n</ng-container>\n<div *ngIf=\"isInEditMode\"\n     [attr.data-cq-data-path]=\"placeholderPath\"\n     [class]=\"getPlaceholderClassNames()\"></div>\n" }]
        }] });

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
class AEMRemoteComponent {
    constructor() { }
    setDataPath(e) {
        this.cqPath = e.cqPath;
    }
}
AEMRemoteComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMRemoteComponent, deps: [], target: i0.ɵɵFactoryTarget.Component });
AEMRemoteComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.4.0", type: AEMRemoteComponent, selector: "aem-remote-component", inputs: { pagePath: "pagePath", itemPath: "itemPath", cqPath: "cqPath" }, ngImport: i0, template: "<aem-model-provider\n[pagePath]=\"pagePath\"\n[itemPath]=\"itemPath\"\n(updateDataPath)=\"setDataPath($event)\"\n[attr.data-cq-data-path]=\"cqPath\"></aem-model-provider>\n", components: [{ type: AEMModelProviderComponent, selector: "aem-model-provider,[aemModelProvider]", inputs: ["cqPath", "cqItem", "itemName", "aemModelProvider", "pagePath", "itemPath"], outputs: ["updateDataPath"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AEMRemoteComponent, decorators: [{
            type: Component,
            args: [{ selector: 'aem-remote-component', template: "<aem-model-provider\n[pagePath]=\"pagePath\"\n[itemPath]=\"itemPath\"\n(updateDataPath)=\"setDataPath($event)\"\n[attr.data-cq-data-path]=\"cqPath\"></aem-model-provider>\n" }]
        }], ctorParameters: function () { return []; }, propDecorators: { pagePath: [{
                type: Input
            }], itemPath: [{
                type: Input
            }], cqPath: [{
                type: Input
            }] } });

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
class SpaAngularEditableComponentsModule {
}
SpaAngularEditableComponentsModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: SpaAngularEditableComponentsModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
SpaAngularEditableComponentsModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: SpaAngularEditableComponentsModule, declarations: [AEMContainerComponent,
        AEMAllowedComponentsContainerComponent,
        AEMResponsiveGridComponent,
        AEMComponentDirective,
        AEMModelProviderComponent,
        AEMPageComponent,
        AEMRemoteComponent], imports: [CommonModule], exports: [AEMContainerComponent,
        AEMAllowedComponentsContainerComponent,
        AEMResponsiveGridComponent,
        AEMComponentDirective,
        AEMModelProviderComponent,
        AEMPageComponent,
        AEMRemoteComponent] });
SpaAngularEditableComponentsModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: SpaAngularEditableComponentsModule, imports: [[
            CommonModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: SpaAngularEditableComponentsModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        CommonModule
                    ],
                    declarations: [
                        AEMContainerComponent,
                        AEMAllowedComponentsContainerComponent,
                        AEMResponsiveGridComponent,
                        AEMComponentDirective,
                        AEMModelProviderComponent,
                        AEMPageComponent,
                        AEMRemoteComponent
                    ],
                    exports: [
                        AEMContainerComponent,
                        AEMAllowedComponentsContainerComponent,
                        AEMResponsiveGridComponent,
                        AEMComponentDirective,
                        AEMModelProviderComponent,
                        AEMPageComponent,
                        AEMRemoteComponent
                    ]
                }]
        }] });

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
class AemPageDataResolver {
    constructor() { }
    /**
     * Returns the absolute resource path without extension.
     * @example
     * // returns: '/content/aa/bb' for route.url [ 'content', 'aa', 'bb.html' ]
     * resolve(route)
     * @param route - route
     * @returns absolute resource path without extension
     */
    resolve(route) {
        return '/' + route.url.join('/').replace(/\.[^/.]+$/, '');
    }
}
AemPageDataResolver.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AemPageDataResolver, deps: [], target: i0.ɵɵFactoryTarget.Injectable });
AemPageDataResolver.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AemPageDataResolver });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.4.0", ngImport: i0, type: AemPageDataResolver, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return []; } });

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
/* eslint-disable @typescript-eslint/no-unused-vars */
/**
 * Implements RouteReuseStrategy to customize route reuse.
 */
class AemPageRouteReuseStrategy {
    /** Determines if this route (and its subtree) should be detached to be reused later. */
    shouldDetach(route) {
        return false;
    }
    /** Not storing deteached route. */
    store(route, detachedTree) { }
    /** Determines if this route (and its subtree) should be reattached. */
    shouldAttach(route) {
        return false;
    }
    /** Retrieves the previously stored route. */
    retrieve(route) {
        return null;
    }
    /** Determines if a route should be reused */
    shouldReuseRoute(future, curr) {
        return false;
    }
}

/*
 * Copyright 2020 Adobe. All rights reserved.
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR REPRESENTATIONS
 * OF ANY KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AEMAllowedComponentsContainerComponent, AEMComponentDirective, AEMContainerComponent, AEMModelProviderComponent, AEMPageComponent, AEMRemoteComponent, AEMResponsiveGridComponent, ALLOWED_COMPONENT_PLACEHOLDER_CLASS_NAMES, ALLOWED_COMPONENT_TITLE_CLASS_NAMES, ALLOWED_PLACEHOLDER_CLASS_NAMES, AbstractMappedComponentDirective, AemPageDataResolver, AemPageRouteReuseStrategy, componentMapping as ComponentMapping, ComponentMappingWithConfig, Constants, LazyMapTo, MapTo, SpaAngularEditableComponentsModule, Utils };
//# sourceMappingURL=adobe-aem-angular-editable-components.mjs.map
